"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const app_configuration_1 = require("../app/app-configuration");
const _viewFunctionSource = {
    redirectTo(route, condition) {
        this._settings.redirect.route = route;
        this._settings.redirect.condition = condition;
    },
    theme(color) {
        this._settings.theme = color;
    }
};
const _appFunctionSource = {
    title(titleName) {
        this._settings.title = titleName;
    },
    settings(options) {
        this._settings.configuration = options;
    },
    routes(routes) {
        this._settings.routes = routes;
    },
    viewTransition(model) {
        model.topToTop !== undefined && Object.assign(this._settings.viewTransition.topToTop, model.topToTop);
        model.topToChild !== undefined && Object.assign(this._settings.viewTransition.topToChild, model.topToChild);
        model.childToTop !== undefined && Object.assign(this._settings.viewTransition.childToTop, model.childToTop);
    }
};
const _findSourceFunctions = function (configParent, objectParent) {
    let functionCollection = new Map();
    for (const key of Object.keys(configParent)) {
        const property = configParent[key];
        if (typeof property !== "object" || property instanceof Array) {
            functionCollection.set(objectParent[key], [property]);
            continue;
        }
        functionCollection = new Map([
            ...functionCollection,
            ..._findSourceFunctions(property, objectParent[key])
        ]);
    }
    return functionCollection;
};
exports.onlyIf = function (callback, elseCallbackName) {
    return function (target, propertyKey, descriptor) {
        const fOldValue = descriptor.value;
        const sElseCallback = elseCallbackName || propertyKey + "Else";
        descriptor.value = function () {
            if (!callback.call(this, this, propertyKey)) {
                this[sElseCallback] && this[sElseCallback].call(this);
                return;
            }
            return fOldValue.apply(this, arguments);
        };
        return descriptor;
    };
};
exports.onlyIf.hasParameters = function (...args) {
    return function (target, propertyKey, descriptor) {
        const fOldValue = descriptor.value;
        const sValue = fOldValue.toLocaleString();
        const aParamsNames = sValue.substring(sValue.indexOf("(") + 1, sValue.indexOf(")")).split(", ");
        const params = {};
        args.forEach((sParamName) => {
            params[sParamName] = aParamsNames.indexOf(sParamName);
        });
        descriptor.value = function () {
            let abort = false;
            for (const key in params) {
                if (arguments[params[key]] !== undefined) {
                    continue;
                }
                abort = true;
            }
            if (abort) {
                return;
            }
            return fOldValue.apply(this, arguments);
        };
        return descriptor;
    };
};
exports.onlyIf.hasProperties = function (...args) {
    return function (target, propertyKey, descriptor) {
        const oldValue = descriptor.value;
        descriptor.value = function () {
            let abort = false;
            args.forEach((arg) => {
                if (this[arg] !== undefined) {
                    return;
                }
                abort = true;
            });
            if (abort) {
                return;
            }
            return oldValue.apply(this, arguments);
        };
        return descriptor;
    };
};
exports.onlyIf.isOnline = function (target, propertyKey, descriptor) {
    const oldValue = descriptor.value;
    const elseCallback = descriptor.offlineCallbackName || `${propertyKey}Offline`;
    descriptor.value = function () {
        if (!navigator.onLine) {
            this[elseCallback] && this[elseCallback].call(this);
            return;
        }
        return oldValue.apply(this, arguments);
    };
    return descriptor;
};
exports.onlyIf.underDevelopment = function (target, propertyKey, descriptor) {
    const oldValue = descriptor.value;
    descriptor.value = function () {
        if (!app_configuration_1.PitayaAppConfiguration.Development.enabled) {
            return;
        }
        return oldValue.apply(this);
    };
    return descriptor;
};
exports.view = function (config) {
    const functionsMap = _findSourceFunctions(config, _viewFunctionSource);
    return function (constructor) {
        return class extends constructor {
            constructor() {
                super();
                for (const [decorator, paramsCollection] of functionsMap) {
                    decorator.call(this, ...paramsCollection.flat());
                }
            }
        };
    };
};
exports.view.redirectTo = function (route, condition) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                _viewFunctionSource.redirectTo.call(this, route, condition);
                return;
            }
        };
    };
};
exports.view.theme = function (color) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                _viewFunctionSource.theme.call(this, color);
            }
        };
    };
};
exports.app = function (config) {
    const functionsMap = _findSourceFunctions(config, _appFunctionSource);
    return function (constructor) {
        return class extends constructor {
            constructor() {
                super();
                for (const [decorator, paramsCollection] of functionsMap) {
                    decorator.call(this, ...paramsCollection.flat());
                }
            }
        };
    };
};
exports.app.title = function (titleName) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                _appFunctionSource.title.call(this, titleName);
            }
        };
    };
};
exports.app.configure = function (options) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                _appFunctionSource.settings.call(this, options);
            }
        };
    };
};
exports.app.routes = function (routes) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                _appFunctionSource.routes.call(this, routes);
            }
        };
    };
};
exports.app.viewTransition = function (model) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                _appFunctionSource.viewTransition.call(this, model);
            }
        };
    };
};
//# sourceMappingURL=decorators.js.map